@@ -26,7 +26,7 @@ sub pmc_support {
     my @pmcs = glob('*.pmc');
     File::Find::find( sub {
         push @pmcs, $File::Find::name if /\.pmc$/i;
-    }, 'lib', 't');
+    }, 't', 'lib');
 
     $self->realclean_files("@pmcs");
 
